package com.lantone.qc.kernel.catalogue.behospitalized;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.Map;


/**
 * @ClassName : BEH0414
 * @Description :  入院记录年龄填写错误
 * @Author : 楼辉荣
 * @Date: 2020-03-06 17:28
 */
@Component
public class BEH0414 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        if (inputInfo.getBeHospitalizedDoc() == null || inputInfo.getFirstPageRecordDoc() == null) {
            return;
        }
        Map<String, String> structureMap = inputInfo.getBeHospitalizedDoc().getStructureMap();
        Map<String, String> firstStructMap = inputInfo.getFirstPageRecordDoc().getStructureMap();
        String contactAddress_bh = structureMap.get("年龄");
        String contactAddress_first = firstStructMap.get(Content.age);
        if (StringUtil.isNotBlank(contactAddress_bh) && StringUtil.isNotBlank(contactAddress_first)) {
            contactAddress_bh = removeUnit(contactAddress_bh);
            contactAddress_first = removeUnit(contactAddress_first);
            if (!contactAddress_first.equals(contactAddress_bh)) {
                status.set("-1");
            }
        }
    }

    private String removeUnit(String str) {
        if (str.contains("岁")) {
            return str.replace("岁", "");
        }
        return str;
    }
}